@@ -64,10 +64,11 @@ typedef struct
 
   gboolean autohide;
   gboolean autohide_missing;
+  gboolean square_icon;
 
   int size;
   GtkOrientation orientation;
-
+  
   GdkPixbuf *pb[7];
 
   GtkWidget *box;
@@ -78,7 +79,7 @@ typedef struct
   GtkTooltips *tooltips;
 
   XfcePanelPlugin *plugin;
-
+  
 } t_wavelan;
 
 static GdkPixbuf *
@@ -88,7 +89,7 @@ load_and_scale(const guint8 *data, int dstw, int dsth)
   int pb_w, pb_h;
 
   TRACE ("Entered load_and_scale");
-
+  
   pb = gdk_pixbuf_new_from_inline(-1, data, FALSE, NULL);
   pb_w = gdk_pixbuf_get_width(pb);
   pb_h = gdk_pixbuf_get_height(pb);
@@ -109,7 +110,7 @@ load_and_scale(const guint8 *data, int dstw, int dsth)
 static void
 wavelan_load_pixbufs(t_wavelan *wavelan)
 {
-  int n, w, h;
+  int n, w, h, second_dim;
 
   TRACE ("Entered wavelan_load_pixbufs, size = %d", wavelan->size);
 
@@ -121,16 +122,26 @@ wavelan_load_pixbufs(t_wavelan *wavelan)
       g_object_unref(G_OBJECT(wavelan->pb[n]));
   }
 
+  /*
+   * Make it square if desired.
+   */
+  if (wavelan->square_icon) {
+    second_dim = wavelan->size;
+  }
+  else {
+    second_dim = -1;
+  }
+
   /*
    * Determine dimension
    */
   if (wavelan->orientation == GTK_ORIENTATION_HORIZONTAL) {
-    w = -1;
+    w = second_dim;
     h = wavelan->size;
   }
   else {
     w = wavelan->size;
-    h = -1;
+    h = second_dim;
   }
 
   /*
@@ -147,13 +158,13 @@ wavelan_load_pixbufs(t_wavelan *wavelan)
 
 static void
 wavelan_set_state(t_wavelan *wavelan, guint state)
-{
+{  
   /* this is OK, happens if this function is called too early */
   if (wavelan->pb[0] == NULL)
     return;
 
   TRACE ("Entered wavelan_set_state, state = %u", state);
-
+  
   if (state > STATE_LINK5)
     state = STATE_LINK5;
 
@@ -176,7 +187,7 @@ wavelan_timer(gpointer data)
   t_wavelan *wavelan = (t_wavelan *)data;
 
   TRACE ("Entered wavelan_timer");
-
+  
   if (wavelan->device != NULL) {
     int result;
 
@@ -220,7 +231,7 @@ wavelan_timer(gpointer data)
 
   /* activate new tooltip */
   if (tip != NULL) {
-    gtk_tooltips_set_tip(wavelan->tooltips, GTK_WIDGET (wavelan->plugin),
+    gtk_tooltips_set_tip(wavelan->tooltips, GTK_WIDGET (wavelan->plugin), 
                          tip, NULL);
     g_free(tip);
   }
@@ -233,7 +244,7 @@ static void
 wavelan_reset(t_wavelan *wavelan)
 {
   TRACE ("Entered wavelan_reset");
-
+  
   if (wavelan->timer_id != 0) {
     g_source_remove(wavelan->timer_id);
     wavelan->timer_id = 0;
@@ -254,15 +265,46 @@ wavelan_reset(t_wavelan *wavelan)
   }
 }
 
+/* query installed devices */
+static GList*
+wavelan_query_interfaces (void)
+{
+  GList *interfaces = NULL;
+  gchar  line[1024];
+  FILE  *fp;
+  gint   n;
+
+  TRACE ("Entered wavelan_query_interface");
+  
+  fp = popen ("/sbin/ifconfig -a", "r");
+  if (fp != NULL)
+    {
+      while (fgets (line, 1024, fp) != NULL)
+        {
+          if (!isalpha (*line))
+            continue;
+
+          for (n = 0; isalnum (line[n]); ++n);
+          line[n] = '\0';
+
+          interfaces = g_list_append (interfaces, g_strdup (line));
+        }
+
+      pclose (fp);
+    }
+
+  return interfaces;
+}
+
 static void
 wavelan_read_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
 {
   char *file;
   XfceRc *rc;
   const char *s;
-
+  
   TRACE ("Entered wavelan_read_config");
-
+  
   if ((file = xfce_panel_plugin_lookup_rc_file (plugin)) != NULL)
   {
     rc = xfce_rc_simple_open (file, TRUE);
@@ -270,46 +312,55 @@ wavelan_read_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
 
     if (rc != NULL)
     {
-      if ((s = xfce_rc_read_entry (rc, "Interface", NULL)) != NULL)
+      if ((s = xfce_rc_read_entry (rc, "Interface", NULL)) != NULL) 
       {
         wavelan->interface = g_strdup (s);
-      }
-
+      } 
+      
       wavelan->autohide = xfce_rc_read_bool_entry (rc, "Autohide", FALSE);
       wavelan->autohide_missing = xfce_rc_read_bool_entry(rc, "AutohideMissing", FALSE);
+      wavelan->square_icon = xfce_rc_read_bool_entry(rc, "SquareIcon", FALSE);
     }
   }
 
+  if (wavelan->interface == NULL) {
+    GList *interfaces = wavelan_query_interfaces();
+    wavelan->interface = g_list_first(interfaces)->data;
+    g_list_free(interfaces);
+  }
+  
   wavelan_reset(wavelan);
 }
 
 static t_wavelan *
 wavelan_new(XfcePanelPlugin *plugin)
 {
-  t_wavelan *wavelan;
+	t_wavelan *wavelan;
   XfceScreenPosition screen_position;
-
+  
   TRACE ("Entered wavelan_new");
-
-  wavelan = g_new0(t_wavelan, 1);
+  
+	wavelan = g_new0(t_wavelan, 1);
 
   wavelan->autohide = FALSE;
   wavelan->autohide_missing = FALSE;
 
-  wavelan->plugin = plugin;
+  wavelan->square_icon = FALSE;
 
+  wavelan->plugin = plugin;
+  
   wavelan->size = xfce_panel_plugin_get_size (plugin);
   screen_position = xfce_panel_plugin_get_screen_position (plugin);
   wavelan->orientation = xfce_panel_plugin_get_orientation (plugin);
-
+ 
   wavelan->ebox = gtk_event_box_new();
   gtk_container_add (GTK_CONTAINER (plugin), wavelan->ebox);
-
+  
   if (xfce_screen_position_is_horizontal (screen_position))
       wavelan->box = xfce_hvbox_new (GTK_ORIENTATION_HORIZONTAL, FALSE, 0);
   else
       wavelan->box = xfce_hvbox_new (GTK_ORIENTATION_VERTICAL, FALSE, 0);
-
+  
   gtk_widget_show(wavelan->box);
   gtk_container_add (GTK_CONTAINER (wavelan->ebox), wavelan->box);
 
@@ -317,10 +368,10 @@ wavelan_new(XfcePanelPlugin *plugin)
   gtk_widget_show (wavelan->button);
 	gtk_box_pack_start(GTK_BOX (wavelan->box), wavelan->button, TRUE, TRUE, 0);*/
   gtk_widget_set_size_request (wavelan->box, -1, -1);
-
+  
   xfce_panel_plugin_add_action_widget (plugin, wavelan->box);
 
-
+  
   wavelan->image = gtk_image_new();
 	gtk_widget_show(wavelan->image);
   gtk_container_add (GTK_CONTAINER (wavelan->box), wavelan->image);
@@ -331,13 +382,13 @@ wavelan_new(XfcePanelPlugin *plugin)
   gtk_object_sink (GTK_OBJECT (wavelan->tooltips));
 
   wavelan_load_pixbufs(wavelan);
-
+  
   wavelan_read_config(plugin, wavelan);
 
   wavelan_set_state(wavelan, wavelan->state);
 
   gtk_widget_show_all(wavelan->ebox);
-
+  
   return(wavelan);
 }
 
@@ -347,7 +398,7 @@ wavelan_free(t_wavelan *wavelan)
   int n;
 
   TRACE ("Entered wavelan_free");
-
+  
   /* free tooltips */
   g_object_unref(G_OBJECT(wavelan->tooltips));
 
@@ -374,16 +425,16 @@ wavelan_write_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
 {
   char *file;
   XfceRc *rc;
-
+  
   TRACE ("Entered wavelan_write_config");
-
+  
   if (!(file = xfce_panel_plugin_save_location (plugin, TRUE)))
   {
     return;
   }
-
+   
   rc = xfce_rc_simple_open (file, FALSE);
-
+  
   g_free (file);
 
   if (!rc)
@@ -395,9 +446,10 @@ wavelan_write_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
   }
   xfce_rc_write_bool_entry (rc, "Autohide", wavelan->autohide);
   xfce_rc_write_bool_entry (rc, "AutohideMissing", wavelan->autohide_missing);
+  xfce_rc_write_bool_entry (rc, "SquareIcon", wavelan->square_icon);
 
   xfce_rc_close(rc);
-
+  
 }
 
 static void
@@ -415,6 +467,14 @@ wavelan_set_size(t_wavelan *wavelan, int size)
   gtk_widget_set_size_request (wavelan->box, -1, -1);
 }
 
+static void
+wavelan_set_square_icon(t_wavelan *wavelan, gboolean square_icon)
+{
+  wavelan->square_icon = square_icon;
+  wavelan_load_pixbufs(wavelan);
+  gtk_widget_set_size_request (wavelan->box, -1, -1);
+}
+
 /* interface changed callback */
 static void
 wavelan_interface_changed(GtkEntry *entry, t_wavelan *wavelan)
@@ -435,7 +495,7 @@ wavelan_autohide_changed(GtkToggleButton *button, t_wavelan *wavelan)
 }
 
 /* autohide on missing callback */
-static void
+static void 
 wavelan_autohide_missing_changed(GtkToggleButton *button, t_wavelan *wavelan)
 {
   TRACE ("Entered wavelan_autohide_missing_changed");
@@ -443,35 +503,12 @@ wavelan_autohide_missing_changed(GtkToggleButton *button, t_wavelan *wavelan)
   wavelan_set_state(wavelan, wavelan->state);
 }
 
-/* query installed devices */
-static GList*
-wavelan_query_interfaces (void)
+/* square icon callback */
+static void 
+wavelan_square_icon_changed(GtkToggleButton *button, t_wavelan *wavelan)
 {
-  GList *interfaces = NULL;
-  gchar  line[1024];
-  FILE  *fp;
-  gint   n;
-
-  TRACE ("Entered wavelan_query_interface");
-
-  fp = popen ("/sbin/ifconfig -a", "r");
-  if (fp != NULL)
-    {
-      while (fgets (line, 1024, fp) != NULL)
-        {
-          if (!isalpha (*line))
-            continue;
-
-          for (n = 0; isalnum (line[n]); ++n);
-          line[n] = '\0';
-
-          interfaces = g_list_append (interfaces, g_strdup (line));
-        }
-
-      pclose (fp);
-    }
-
-  return interfaces;
+  TRACE ("Entered wavelan_square_icon_changed");
+  wavelan_set_square_icon(wavelan, gtk_toggle_button_get_active(button));
 }
 
 static void
@@ -489,15 +526,15 @@ static void
 wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 {
   GtkWidget *dlg, *hbox, *label, *interface, *vbox, *autohide;
-  GtkWidget *autohide_missing, *header, *warn_label;
+  GtkWidget *autohide_missing, *header, *warn_label, *square_icon;
   GtkWidget *combo;
   GList     *interfaces, *lp;
 
   TRACE ("Entered wavelan_create_options");
-
+  
   dlg = gtk_dialog_new_with_buttons (_("Properties"),
               GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
-              GTK_DIALOG_DESTROY_WITH_PARENT |
+              GTK_DIALOG_DESTROY_WITH_PARENT | 
               GTK_DIALOG_NO_SEPARATOR,
               GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
               NULL);
@@ -508,7 +545,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 
   g_signal_connect (dlg, "response", G_CALLBACK (wavelan_dialog_response),
                     wavelan);
-
+  
   gtk_container_set_border_width (GTK_CONTAINER (dlg), 2);
 
   header = xfce_create_header (NULL, _("Wavelan Plugin Options"));
@@ -517,16 +554,16 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_widget_show (header);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), header,
                       FALSE, TRUE, 0);
-
+              
   vbox = gtk_vbox_new(FALSE, 8);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_widget_show(vbox);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), vbox,
                       TRUE, TRUE, 0);
-
+  
   hbox = gtk_hbox_new(FALSE, 8);
   gtk_widget_show(hbox);
-
+  
   label = gtk_label_new(_("Interface"));
   gtk_widget_show(label);
 
@@ -556,13 +593,13 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_widget_show(autohide);
   gtk_box_pack_start(GTK_BOX(hbox), autohide, TRUE, TRUE, 1);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
-
+  
   hbox = gtk_hbox_new(FALSE, 2);
   gtk_widget_show(hbox);
   autohide_missing = gtk_check_button_new_with_label(_("Autohide when no hardware present"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide_missing),
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide_missing), 
       wavelan->autohide_missing);
-  g_signal_connect(autohide_missing, "toggled",
+  g_signal_connect(autohide_missing, "toggled", 
       G_CALLBACK(wavelan_autohide_missing_changed), wavelan);
   gtk_widget_show(autohide_missing);
   gtk_box_pack_start(GTK_BOX(hbox), autohide_missing, TRUE, TRUE, 1);
@@ -576,21 +613,32 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), warn_label, TRUE, TRUE, 1);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
 
+  hbox = gtk_hbox_new(FALSE, 2);
+  gtk_widget_show(hbox);
+  square_icon = gtk_check_button_new_with_label(_("Use a square icon"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(square_icon), 
+      wavelan->square_icon);
+  g_signal_connect(square_icon, "toggled", 
+      G_CALLBACK(wavelan_square_icon_changed), wavelan);
+  gtk_widget_show(square_icon);
+  gtk_box_pack_start(GTK_BOX(hbox), square_icon, TRUE, TRUE, 1);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+
   for (lp = interfaces; lp != NULL; lp = lp ->next)
     g_free (lp->data);
   g_list_free (interfaces);
 
   gtk_widget_show (dlg);
-
+  
 }
 
 static void
-wavelan_set_screen_position (t_wavelan *wavelan,
+wavelan_set_screen_position (t_wavelan *wavelan, 
                              XfceScreenPosition position)
 {
 
   /* Do I really need anything here? */
-
+  
 }
 
 /*static void
@@ -598,7 +646,7 @@ wavelan_screen_position_changed (XfcePanelPlugin *plugin,
                                  XfceScreenPosition position,
                                  t_wavelan *wavelan)
 {
-    wavelan_set_screen_postion (wavelan, position);
+    wavelan_set_screen_postion (wavelan, position); 
 }*/
 
 static void
@@ -608,7 +656,7 @@ wavelan_orientation_changed (XfcePanelPlugin *plugin,
 {
     wavelan_set_orientation(wavelan, orientation);
 }
-
+  
 
 static void
 wavelan_size_changed(XfcePanelPlugin *plugin,
@@ -618,7 +666,7 @@ wavelan_size_changed(XfcePanelPlugin *plugin,
     wavelan_set_size(wavelan, size);
 }
 
-static void
+static void 
 wavelan_free_data (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 {
   wavelan_free(wavelan);
@@ -639,9 +687,9 @@ wavelan_configure (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 static void
 wavelan_construct (XfcePanelPlugin *plugin)
 {
-
+  
   TRACE ("Entered wavelan_construct");
-
+  
   t_wavelan *wavelan = wavelan_new(plugin);
 
 /*  g_signal_connect (plugin, "screen-position-changed",
@@ -652,20 +700,20 @@ wavelan_construct (XfcePanelPlugin *plugin)
 
   g_signal_connect (plugin, "size-changed",
                     G_CALLBACK (wavelan_size_changed), wavelan);
-
+ 
   g_signal_connect (plugin, "free-data",
                     G_CALLBACK (wavelan_free_data), wavelan);
 
   g_signal_connect (plugin, "save",
                     G_CALLBACK (wavelan_save), wavelan);
-
+  
   xfce_panel_plugin_menu_show_configure (plugin);
   g_signal_connect (plugin, "configure-plugin",
                     G_CALLBACK (wavelan_configure), wavelan);
-
+  
   wavelan_set_screen_position (wavelan,
           xfce_panel_plugin_get_screen_position (plugin));
-
+  
 }
 
 XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL(wavelan_construct);
